@@ -1,4 +1,5 @@
-# Copyright 2021 MosaicML. All Rights Reserved.
+# Copyright 2022 MosaicML Composer authors
+# SPDX-License-Identifier: Apache-2.0
 
 """Core AugMix classes and functions."""
 
@@ -9,6 +10,7 @@ from typing import List, TypeVar
 
 import numpy as np
 import torch
+import torch.utils.data
 from PIL import Image
 from PIL.Image import Image as PillowImage
 from torchvision.datasets import VisionDataset
@@ -55,7 +57,7 @@ def augmix_image(img: ImgT,
             )
 
     Args:
-        img (PIL.Image.Image or torch.Tensor): Image or batch of images to be AugMix'd.
+        img (PIL.Image.Image | torch.Tensor): Image or batch of images to be AugMix'd.
         severity (int, optional): See :class:`.AugMix`.
         depth (int, optional): See :class:`.AugMix`.
         width (int, optional): See :class:`.AugMix`.
@@ -255,7 +257,12 @@ class AugMix(Algorithm):
         self._transformed_datasets = weakref.WeakSet()
 
     def match(self, event: Event, state: State) -> bool:
-        return event == Event.FIT_START and state.train_dataloader.dataset not in self._transformed_datasets
+        if event != Event.FIT_START:
+            return False
+        assert state.dataloader is not None, "dataloader should be defined on fit start"
+        if not isinstance(state.dataloader, torch.utils.data.DataLoader):
+            raise TypeError(f"{type(self).__name__} requires a PyTorch dataloader.")
+        return state.dataloader.dataset not in self._transformed_datasets
 
     def apply(self, event: Event, state: State, logger: Logger) -> None:
         am = AugmentAndMixTransform(severity=self.severity,
@@ -263,7 +270,8 @@ class AugMix(Algorithm):
                                     width=self.width,
                                     alpha=self.alpha,
                                     augmentation_set=self.augmentation_set)
-        dataset = state.train_dataloader.dataset
+        assert isinstance(state.dataloader, torch.utils.data.DataLoader), "dataloader type checked on match()"
+        dataset = state.dataloader.dataset
         if not isinstance(dataset, VisionDataset):
             raise TypeError(
                 textwrap.dedent(f"""\
