@@ -1,4 +1,5 @@
-# Copyright 2021 MosaicML. All Rights Reserved.
+# Copyright 2022 MosaicML Composer authors
+# SPDX-License-Identifier: Apache-2.0
 
 """Core code for sequence length warmup."""
 
@@ -7,8 +8,10 @@ from math import ceil
 from typing import Dict, Mapping, Optional
 
 import torch
+import torch.utils.data
 
 from composer.core import Algorithm, Event, State
+from composer.core.precision import get_precision_context
 from composer.core.time import TimeUnit
 from composer.core.types import Batch
 from composer.loggers import Logger
@@ -179,12 +182,12 @@ class SeqLengthWarmup(Algorithm):
                     {type(self).__name__} requires state.model to be of type {ComposerTransformer.__name__}, not of type {type(state.model)}"""
                                    ))
 
-            if state.train_dataloader.batch_size is None:
-                raise RuntimeError("Sequence Length Warmup algorithm requires constant batch size.")
-
             self._original_model = state.model
             return
 
+        assert state.dataloader is not None, "dataloader should be set on AFTER_DATALOADER"
+        assert state.max_duration is not None, "max_duration should be set on AFTER_DATALOADER"
+
         # in order to avoid OOMs, we do a forward and a backward pass on a dummy input.
         if not self._activated:
             # ensure that input_ids is a valid model input. since we don't need the
@@ -204,7 +207,14 @@ class SeqLengthWarmup(Algorithm):
             # all of the parameters
             device = next(state.model.parameters()).device
 
-            per_gpu_macrobatch = state.train_dataloader.batch_size
+            try:
+                # Both PyTorch and FFCV dataloaders define a `batch_size` attribute
+                # This exception would mainly be raised if the user is passing in a custom
+                # iterable
+                per_gpu_macrobatch = getattr(state.dataloader, "batch_size")
+            except AttributeError as e:
+                raise AttributeError(
+                    "Sequence Length Warmup requires the `state.dataloader` to have a `batch_size` attribute.") from e
             if per_gpu_macrobatch is None:
                 raise RuntimeError("Sequence Length Warmup algorithm requires constant batch size.")
             per_gpu_batch = ceil(per_gpu_macrobatch / state.grad_accum)
@@ -223,7 +233,7 @@ class SeqLengthWarmup(Algorithm):
 
             # start by running a forward and backward pass
             # of the maximum sequence length to allocate cache.
-            with state.precision_context:
+            with get_precision_context(state.precision):
                 outputs = state.model.forward(model_inputs)
                 loss = self._original_model.loss(outputs, model_inputs)
 
@@ -238,7 +248,9 @@ class SeqLengthWarmup(Algorithm):
             self._activated = True
 
         if state.max_duration.unit == TimeUnit.EPOCH:
-            num_optimization_steps = state.steps_per_epoch * state.max_duration.value
+            if state.dataloader_len is None:
+                raise RuntimeError("Sequential Length Warmup requires the dataloader to be sized.")
+            num_optimization_steps = int(state.dataloader_len) * state.max_duration.value
         elif state.max_duration.unit == TimeUnit.BATCH:
             num_optimization_steps = state.max_duration.value
         else:
@@ -254,13 +266,13 @@ class SeqLengthWarmup(Algorithm):
         num_update_steps = (self.max_seq_length - self.min_seq_length) // self.step_size
         update_every_n_steps = num_warmup_steps // num_update_steps
 
-        curr_seq_len = self.step_size * (int(state.timer.batch) // update_every_n_steps)
+        curr_seq_len = self.step_size * (int(state.timestamp.batch) // update_every_n_steps)
         curr_seq_len = max(curr_seq_len, self.min_seq_length)
         curr_seq_len = min(curr_seq_len, self.max_seq_length)
 
-        state.batch = set_batch_sequence_length(state.batch_dict, curr_seq_len, self.truncate)
+        state.batch = set_batch_sequence_length(state.batch, curr_seq_len, self.truncate)
 
-        batch_size = state.batch_dict['input_ids'].shape[0]
+        batch_size = state.batch['input_ids'].shape[0]
         logger.data_batch({
             'seq_length_warmup/curr_seq_len': curr_seq_len,
             'seq_length_warmup/curr_bs': batch_size,
