@@ -1,4 +1,5 @@
-# Copyright 2021 MosaicML. All Rights Reserved.
+# Copyright 2022 MosaicML Composer authors
+# SPDX-License-Identifier: Apache-2.0
 
 """Hyperparameters for callbacks."""
 from __future__ import annotations
@@ -6,16 +7,18 @@ from __future__ import annotations
 import abc
 import textwrap
 from dataclasses import asdict, dataclass
-from typing import Optional
+from typing import Optional, Union
 
 import yahp as hp
 
 from composer.callbacks.checkpoint_saver import CheckpointSaver
+from composer.callbacks.early_stopper import EarlyStopper
 from composer.callbacks.grad_monitor import GradMonitor
 from composer.callbacks.lr_monitor import LRMonitor
 from composer.callbacks.memory_monitor import MemoryMonitor
 from composer.callbacks.mlperf import MLPerfCallback
 from composer.callbacks.speed_monitor import SpeedMonitor
+from composer.callbacks.threshold_stopper import ThresholdStopper
 from composer.core.callback import Callback
 from composer.core.time import Time
 from composer.utils import import_object
@@ -51,6 +54,7 @@ class GradMonitorHparams(CallbackHparams):
     Args:
         log_layer_grad_norms (bool, optional):
             See :class:`~.GradMonitor` for documentation.
+            Default: ``False``.
     """
 
     log_layer_grad_norms: bool = hp.optional(
@@ -120,6 +124,88 @@ class SpeedMonitorHparams(CallbackHparams):
         return SpeedMonitor(window_size=self.window_size)
 
 
+@dataclass
+class EarlyStopperHparams(CallbackHparams):
+    """:class:`~.EarlyStopper` hyperparameters.
+
+    Args:
+        monitor (str): The name of the metric to monitor.
+        dataloader_label (str): The label of the dataloader or evaluator associated with the tracked metric. If 
+            monitor is in an Evaluator, the dataloader_label field should be set to the Evaluator's label. If 
+            monitor is a training metric or an ordinary evaluation metric not in an Evaluator, dataloader_label
+            should be set to 'train' or 'eval' respectively.
+        comp (str, optional): A string dictating which comparison operator to use to measure
+            change in the monitored metric. Set ``comp`` to "less" to use the function :func:`torch.less`,
+            and "greater" to use the function :func:`torch.greater`. The comparison operator will be called
+            ``comp(current_value, prev_best)``. For example, for metrics where the optimal value is low
+            (error, loss, perplexity), use a less than operator.
+        min_delta (float, optional): An optional float that requires a new value to exceed the best value by at
+            least that amount. Defaults to 0.
+        patience (int | str, optional): The interval of time the monitored metric can not improve without stopping
+            training. Defaults to 1 epoch. If patience is an integer, it is interpreted as the number of epochs.
+    """
+    monitor: str = hp.required("The name of the metric to monitor.")
+    dataloader_label: str = hp.required("Label of the dataloader/evaluator associated with the metric.")
+    comp: Optional[str] = hp.optional("Which comparison operator to use to track change in the metric.", default=None)
+    min_delta: float = hp.optional("New metric value must exceed the best value by min_delta to continue training.",
+                                   default=0.0)
+    patience: Union[int, str] = hp.optional("Interval the trainer can wait without stopping training.", default=1)
+
+    def initialize_object(self) -> EarlyStopper:
+        """Initialize the EarlyStopper callback.
+
+        Returns:
+            EarlyStopper: An instance of :class:`~.EarlyStopper`.
+        """
+        return EarlyStopper(monitor=self.monitor,
+                            dataloader_label=self.dataloader_label,
+                            comp=self.comp,
+                            min_delta=self.min_delta,
+                            patience=self.patience)
+
+
+@dataclass
+class ThresholdStopperHparams(CallbackHparams):
+    """:class:`~.ThresholdStopper` hyperparameters.
+
+    Args:
+        monitor (str): The name of the metric to monitor.
+        dataloader_label (str): The label of the dataloader or evaluator associated with the tracked metric. If 
+            monitor is in an Evaluator, the dataloader_label field should be set to the Evaluator's label. If 
+            monitor is a training metric or an ordinary evaluation metric not in an Evaluator, dataloader_label
+            should be set to 'train' or 'eval' respectively.
+        threshold (float): The threshold that dictates when to halt training. Whether training stops if the metric
+            exceeds or falls below the threshold depends on the comparison operator.
+        comp (str, optional): A string dictating which comparison operator to use to measure
+            change in the monitored metric. Set ``comp`` to "less" to use the function :func:`torch.less`,
+            and "greater" to use the function :func:`torch.greater`. The comparison operator will be called
+            ``comp(current_value, prev_best)``. For example, for metrics where the optimal value is low
+            (error, loss, perplexity), use the less than operator.
+        stop_on_batch (bool, optional): A bool that indicates whether to stop training in the middle of an epoch if
+            the training metrics satisfy the threshold comparison. Defaults to False.
+    """
+    monitor: str = hp.required("The name of the metric to monitor.")
+    dataloader_label: str = hp.required("Label of the dataloader/evaluator associated with the metric.")
+    threshold: float = hp.required("The threshold value to compare the metric to.")
+    comp: Optional[str] = hp.optional("Which comparison operator to use to track change in the metric.", default=None)
+    stop_on_batch: bool = hp.optional("Whether to stop training in the middle of an epoch if using training metrics.",
+                                      default=False)
+
+    def initialize_object(self) -> ThresholdStopper:
+        """Initialize the ThresholdStopper callback.
+
+        Returns:
+            ThresholdStopper: An instance of :class:`~.ThresholdStopper`.
+        """
+        return ThresholdStopper(
+            monitor=self.monitor,
+            dataloader_label=self.dataloader_label,
+            threshold=self.threshold,
+            comp=self.comp,
+            stop_on_batch=self.stop_on_batch,
+        )
+
+
 @dataclass
 class MLPerfCallbackHparams(CallbackHparams):
     """:class:`~.MLPerfCallback` hyperparameters.
@@ -128,20 +214,22 @@ class MLPerfCallbackHparams(CallbackHparams):
         root_folder (str): The root submission folder
         index (int): The repetition index of this run. The filename created will be
             ``result_[index].txt``.
-        benchmark (str, optional): Benchmark name. Currently only ``resnet`` supported.
+        benchmark (str, optional): Benchmark name. Currently only ``resnet`` supported. Default: ``resnet``.
         target (float, optional): The target metric before the mllogger marks the stop
             of the timing run. Default: ``0.759`` (resnet benchmark).
-        division (str, optional): Division of submission. Currently only ``open`` division supported.
-        metric_name (str, optional): name of the metric to compare against the target. Default: ``Accuracy``.
+        division (str, optional): Division of submission. Currently only ``open`` division supported. Default: ``"open"``.
+        metric_name (str, optional): name of the metric to compare against the target. Default: ``"Accuracy"``.
         metric_label (str, optional): label name. The metric will be accessed via ``state.current_metrics[metric_label][metric_name]``.
-        submitter (str, optional): Submitting organization. Default: MosaicML.
+            Default: ``"eval"``.
+        submitter (str, optional): Submitting organization. Default: ``"MosaicML"``.
         system_name (str, optional): Name of the system (e.g. 8xA100_composer). If
-            not provided, system name will default to ``[world_size]x[device_name]_composer``,
-            e.g. ``8xNVIDIA_A100_80GB_composer``.
+            ``None``, system name will default to ``[world_size]x[device_name]_composer``,
+            e.g. ``8xNVIDIA_A100_80GB_composer``. Default: ``None``.
         status (str, optional): Submission status. One of (onprem, cloud, or preview).
             Default: ``"onprem"``.
         cache_clear_cmd (str, optional): Command to invoke during the cache clear. This callback
-            will call ``subprocess(cache_clear_cmd)``. Default is disabled (None)
+            will call ``subprocess(cache_clear_cmd)``. Default is disabled (``None``)
+        host_processors_per_node (int, optional): Total number of host processors per node.  Default: ``None``.
 
     """
 
@@ -165,6 +253,8 @@ class MLPerfCallbackHparams(CallbackHparams):
         "Command to invoke during the cache clear. This callback will call subprocess(cache_clear_cmd). Default: Disabled.",
         default=None,
     )
+    host_processors_per_node: Optional[int] = hp.optional(
+        "Total number of host processors per node.  Default: ``None``.", default=None)
 
     def initialize_object(self) -> MLPerfCallback:
         """Initialize the MLPerf Callback.
@@ -184,19 +274,19 @@ class CheckpointSaverHparams(CallbackHparams):
         filename (str, optional): See :class:`~.CheckpointSaver`.
         artifact_name (str, optional): See :class:`~.CheckpointSaver`.
         latest_filename (str, optional): See :class:`~.CheckpointSaver`.
-        overwrite (str, optional): See :class:`~.CheckpointSaver`.
+        overwrite (bool, optional): See :class:`~.CheckpointSaver`.
+            Default: ``False``.
         weights_only (bool, optional): See :class:`~.CheckpointSaver`.
-        num_checkpoints_to_keep (int, optional): See :class:`~.CheckpointSaver`.
-
+            Deafult: ``False``.
         save_interval (str, optional): Either a :doc:`time-string </trainer/time>` or a path to a function.
-
             If a :doc:`time-string </trainer/time>`, checkpoints will be saved according to this interval.
 
             If a path to a function, it should be of the format ``'path.to.function:function_name'``. The function
             should take (:class:`~.State`, :class:`~.Event`) and return a
             boolean indicating whether a checkpoint should be saved given the current state and event. The event will
-            be either :attr:`~composer.core.event.Event.BATCH_CHECKPOINT` or
-            :attr:`~composer.core.event.Event.EPOCH_CHECKPOINT`.
+            be either :attr:`.Event.BATCH_CHECKPOINT` or :attr:`.Event.EPOCH_CHECKPOINT`. Default: ``"1ep"``.
+        num_checkpoints_to_keep (int, optional): See :class:`~.CheckpointSaver`.
+            Default: ``-1``.
     """
     save_folder: str = hp.optional(doc="Folder where checkpoints will be saved.", default="{run_name}/checkpoints")
     filename: str = hp.optional("Checkpoint name format string.", default="ep{epoch}-ba{batch}-rank{rank}")
@@ -205,7 +295,7 @@ class CheckpointSaverHparams(CallbackHparams):
     latest_filename: Optional[str] = hp.optional("Latest checkpoint symlink format string.",
                                                  default="latest-rank{rank}")
     overwrite: bool = hp.optional("Whether to override existing checkpoints.", default=False)
-    weights_only: bool = hp.optional("Whether to save only checkpoint weights", default=False)
+    weights_only: bool = hp.optional("Whether to save only checkpoint weights.", default=False)
     save_interval: str = hp.optional(textwrap.dedent("""\
         Checkpoint interval or path to a `(State, Event) -> bool` function
         returning whether a checkpoint should be saved."""),
