@@ -1,4 +1,5 @@
-# Copyright 2021 MosaicML. All Rights Reserved.
+# Copyright 2022 MosaicML Composer authors
+# SPDX-License-Identifier: Apache-2.0
 
 """Callback to save checkpoints during training."""
 
@@ -17,8 +18,8 @@ from composer.loggers import Logger
 from composer.loggers.logger import LogLevel
 from composer.utils import checkpoint, dist
 from composer.utils.file_helpers import (FORMAT_NAME_WITH_DIST_AND_TIME_TABLE, FORMAT_NAME_WITH_DIST_TABLE,
-                                         ensure_folder_is_empty, format_name_with_dist, format_name_with_dist_and_time,
-                                         is_tar)
+                                         ensure_folder_has_no_conflicting_files, format_name_with_dist,
+                                         format_name_with_dist_and_time, is_tar)
 
 log = logging.getLogger(__name__)
 
@@ -29,16 +30,16 @@ def checkpoint_periodically(interval: Union[str, int, Time]) -> Callable[[State,
     """Helper function to create a checkpoint scheduler according to a specified interval.
 
     Args:
-        interval (Union[str, int, Time]): The interval describing how often checkpoints should be
-            saved. If an integer, it will be assumed to be in :attr:`~TimeUnit.EPOCH`\\s.
-            Otherwise, the unit must be either :attr:`TimeUnit.EPOCH` or :attr:`TimeUnit.BATCH`.
+        interval (Union[str, int, :class:`.Time`]): The interval describing how often checkpoints should be
+            saved. If an integer, it will be assumed to be in :attr:`.TimeUnit.EPOCH`\\s.
+            Otherwise, the unit must be either :attr:`.TimeUnit.EPOCH` or :attr:`.TimeUnit.BATCH`.
 
             Checkpoints will be saved every ``n`` batches or epochs (depending on the unit),
             and at the end of training.
 
     Returns:
         Callable[[State, Event], bool]: A function that can be passed as the ``save_interval``
-            argument into the :class:`CheckpointSaver`.
+            argument into the :class:`.CheckpointSaver`.
     """
     if isinstance(interval, str):
         interval = Time.from_timestring(interval)
@@ -53,27 +54,30 @@ def checkpoint_periodically(interval: Union[str, int, Time]) -> Callable[[State,
         raise NotImplementedError(
             f"Unknown checkpointing interval: {interval.unit}. Must be TimeUnit.EPOCH or TimeUnit.BATCH.")
 
-    last_checkpoint_batch = None
+    last_checkpoint_batch: Optional[Time] = None
 
     def save_interval(state: State, event: Event):
         nonlocal last_checkpoint_batch
-        if state.get_elapsed_duration() >= 1.0:
+        elapsed_duration = state.get_elapsed_duration()
+        assert elapsed_duration is not None, "elapsed_duration is set on the BATCH_CHECKPOINT and EPOCH_CHECKPOINT"
+
+        if elapsed_duration >= 1.0:
             # if doing batch-wise checkpointing, and we saved a checkpoint at the batch_checkpoint event
             # right before the epoch_checkpoint event, do not save another checkpoint at the epoch_checkpoint
             # event if the batch count didn't increase.
-            if state.timer.batch != last_checkpoint_batch:
-                last_checkpoint_batch = state.timer.batch
+            if state.timestamp.batch != last_checkpoint_batch:
+                last_checkpoint_batch = state.timestamp.batch
                 return True
 
         if save_event == Event.EPOCH_CHECKPOINT:
-            count = state.timer.epoch
+            count = state.timestamp.epoch
         elif save_event == Event.BATCH_CHECKPOINT:
-            count = state.timer.batch
+            count = state.timestamp.batch
         else:
             raise RuntimeError(f"Invalid save_event: {save_event}")
 
         if event == save_event and int(count) % int(interval) == 0:
-            last_checkpoint_batch = state.timer.batch
+            last_checkpoint_batch = state.timestamp.batch
             return True
 
         return False
@@ -86,11 +90,11 @@ class CheckpointSaver(Callback):
 
     .. note::
 
-        If the ``folder`` argument is specified constructing the :class:`~composer.trainer.trainer.Trainer`,
-        then the :class:`.CheckpointSaver` callback need not be constructed manually. However, for advanced
-        checkpointing use cases (such as saving a weights-only checkpoint at one interval and the full training state
+        If the ``folder`` argument is specified when constructing the :class:`.Trainer`, then the :class:`.CheckpointSaver`
+        callback need not be constructed manually. However, for advanced checkpointing use cases 
+        (such as saving a weights-only checkpoint at one interval and the full training state
         at another interval), instance(s) of this :class:`.CheckpointSaver` callback can be specified in the
-        ``callbacks`` argument of the :class:`~composer.trainer.trainer.Trainer`, as shown in the example below.
+        ``callbacks`` argument of the :class:`.Trainer`, as shown in the example below.
 
     Example
 
@@ -109,14 +113,14 @@ class CheckpointSaver(Callback):
         ...         weights_only=False,
         ...     )
         ... ])
-    
+
     .. testcleanup::
 
         trainer.engine.close()
 
     Args:
         folder (str, optional): Format string for the folder where checkpoints will be saved.
-            (default: ``'{{run_name}}/checkpoints'``)
+            Default: ``'{{run_name}}/checkpoints'``.
 
             The following format variables are available:
 
@@ -128,7 +132,7 @@ class CheckpointSaver(Callback):
                 Otherwise, multiple processes may attempt to write to the same file.
 
         filename (str, optional): A format string describing how to name checkpoints.
-            (default: ``'ep{{epoch}}-ba{{batch}}-rank{{rank}}'``)
+            Default: ``'ep{{epoch}}-ba{{batch}}-rank{{rank}}'``.
 
             Checkpoints will be saved approximately to ``{{folder}}/{{filename.format(...)}}``.
 
@@ -140,7 +144,7 @@ class CheckpointSaver(Callback):
             .. note::
 
                 *   By default, only the rank zero process will save a checkpoint file.
-                
+
                 *   When using DeepSpeed, each rank will save a checkpoint file in tarball format. DeepSpeed
                     requires tarball format, as it saves model and optimizer states in separate files.
                     Ensure that ``'{{rank}}'`` appears within the ``filename``. Otherwise, multiple ranks
@@ -150,13 +154,13 @@ class CheckpointSaver(Callback):
                 *   To use compression (regardless of whether DeepSpeed is enabled), set the file extension
                     to ``'.tar.gz'``, ``'.tgz'``, ``'.tar.bzip'``, or ``'.tar.lzma'`` (depending on the desired
                     compression algorithm).
-            
+
             .. warning::
 
                 Using compression will block the training loop while checkpoints are being compressed. As such, we
                 recommend saving checkpoints without compression.
 
-            Consider the following scenario, where:
+            Consider the following scenario where:
 
             *   The :attr:`~.Logger.run_name` is ``'awesome-training-run'``
             *   The default ``folder='{{run_name}}/checkpoints'`` is used.
@@ -164,7 +168,8 @@ class CheckpointSaver(Callback):
             *   The current epoch count is ``1``.
             *   The current batch count is ``42``.
 
-            When DeepSpeed is not being used, the rank zero process will save the checkpoint to ``"awesome-training-run/checkpoints/ep1-ba42-rank0"``.
+            When DeepSpeed is not being used, the rank zero process will save the checkpoint to 
+            ``"awesome-training-run/checkpoints/ep1-ba42-rank0"``.
 
             When DeepSpeed is being used, each rank (process) will save checkpoints to::
 
@@ -172,14 +177,14 @@ class CheckpointSaver(Callback):
                 awesome-training-run/checkpoints/ep1-ba42-rank1.tar
                 awesome-training-run/checkpoints/ep1-ba42-rank2.tar
                 ...
-        
+
         artifact_name (str, optional): Format string for the checkpoint's artifact name.
-            (default: ``'{{run_name}}/checkpoints/ep{{epoch}}-ba{{batch}}-rank{{rank}}"``)
-        
+            Default: ``"{{run_name}}/checkpoints/ep{{epoch}}-ba{{batch}}-rank{{rank}}"``.
+
             After the checkpoint is saved, it will be periodically logged as a file artifact.
             The artifact name will be determined by this format string.
 
-            .. seealso:: :meth:`~composer.loggers.logger.Logger.log_file_artifact` for file artifact logging.
+            .. seealso:: :meth:`.Logger.log_file_artifact` for file artifact logging.
 
             The same format variables for ``filename`` are available.
 
@@ -187,7 +192,7 @@ class CheckpointSaver(Callback):
 
             To disable logging trace files as file artifacts, set this parameter to ``None``.
         latest_filename (str, optional): A format string for a symlink which points to the last saved checkpoint.
-            (default: ``'latest-rank{{rank}}'``)
+            Default: ``'latest-rank{{rank}}'``.
             
             Symlinks will be created approximately at ``{{folder}}/{{latest_filename.format(...)}}``. 
 
@@ -223,12 +228,12 @@ class CheckpointSaver(Callback):
                 awesome-training-run/checkpoints/latest-rank2.tar -> awesome-training-run/checkpoints/ep1-ba42-rank2.tar
                 ...
         latest_artifact_name (str, optional): Format string for the checkpoint's latest symlink artifact name.
-            (default: ``'{{run_name}}/checkpoints/latest-rank{{rank}}"``)
-        
-            Whenever a new checkpoint is saved, a symlink artifact is created or updated to point to the latest checkpoint's ``artifact_name``.
-            The artifact name will be determined by this format string. This parameter has no effect if ``latest_filename`` or ``artifact_name`` is None."
+            Default: ``'{{run_name}}/checkpoints/latest-rank{{rank}}"``.
 
-            .. seealso:: :meth:`~composer.loggers.logger.Logger.log_symlink_artifact` for symlink artifact logging.
+            Whenever a new checkpoint is saved, a symlink artifact is created or updated to point to the latest checkpoint's ``artifact_name``.
+            The artifact name will be determined by this format string. This parameter has no effect if ``latest_filename`` or ``artifact_name`` is ``None``.
+
+            .. seealso:: :meth:`.Logger.log_symlink_artifact` for symlink artifact logging.
 
             The same format variables for ``filename`` are available.
 
@@ -237,35 +242,35 @@ class CheckpointSaver(Callback):
             To disable symlinks in logger, set this parameter to ``None``.
 
         overwrite (bool, optional): Whether existing checkpoints should be overridden.
-            If ``False`` (the default), then the ``folder`` must not exist or be empty.
-            (default: ``False``)
+            If ``False`` (the default), then the ``folder`` must not exist or must not contain checkpoints which may conflict
+            with the current run. Default: ``False``.
 
-        save_interval (Time | str | int | (State, Event) -> bool): A :class:`Time`, time-string, integer (in epochs),
+        save_interval (:class:`.Time` | str | int | (:class:`.State`, :class:`.Event`) -> bool): A :class:`.Time`, time-string, integer (in epochs),
             or a function that takes (state, event) and returns a boolean whether a checkpoint should be saved.
 
             If an integer, checkpoints will be saved every n epochs.
-            If :class:`Time` or a time-string, checkpoints will be saved according to this interval.
+            If :class:`.Time` or a time-string, checkpoints will be saved according to this interval.
 
             .. seealso:: :func:`.checkpoint_periodically`
 
-            If a function, then this function should take two arguments (:class:`State`, :class:`Event`).
+            If a function, then this function should take two arguments (:class:`.State`, :class:`.Event`).
             The first argument will be the current state of the trainer, and the second argument will be
-            be :attr:`.Event.BATCH_CHECKPOINT` or :attr:`.EPOCH_CHECKPOINT` (depending on the current training
+            be :attr:`.Event.BATCH_CHECKPOINT` or :attr:`.Event.EPOCH_CHECKPOINT` (depending on the current training
             progress). It should return ``True`` if a checkpoint should be saved given the current state and
             event.
 
         weights_only (bool): If ``True``, save only the model weights instead of the entire training state.
-            This parmeter must be ``False`` when using DeepSpeed. (default: ``False``)
+            This parmeter must be ``False`` when using DeepSpeed. Default: ``False``.
 
 
         num_checkpoints_to_keep (int, optional): The number of checkpoints to keep locally. The oldest checkpoints
-            are removed first. Set to ``-1`` to keep all checkpoints locally. (default: ``-1``)
+            are removed first. Set to ``-1`` to keep all checkpoints locally. Default: ``-1``.
 
             Checkpoints will be removed after they have been logged as a file artifact. For example, when this callback
-            is used in conjunction with the :class:`~composer.loggers.object_store_logger.ObjectStoreLogger`, set this
+            is used in conjunction with the :class:`.ObjectStoreLogger`, set this
             parameter to ``0`` to immediately delete checkpoints from the local disk after they have been uploaded to
             the object store.
-            
+
             This parameter only controls how many checkpoints are kept locally; checkpoints are not deleted from
             artifact stores.
 
@@ -277,7 +282,7 @@ class CheckpointSaver(Callback):
             will be at the end.
 
             .. note::
-                
+
                 When using DeepSpeed, the index of a filepath in each list corresponds to the global rank of
                 the process that wrote that file. Each filepath is valid only on the process's (rank's) node.
 
@@ -317,12 +322,15 @@ class CheckpointSaver(Callback):
         del state  # unused
         folder = format_name_with_dist(self.folder, logger.run_name)
         os.makedirs(folder, exist_ok=True)
-        if not self.overwrite:
-            ensure_folder_is_empty(folder)
-        # Ensure no rank proceeds (and potentially attempts to write to the folder), until all ranks have validated that the folder is empty.
-        dist.barrier()
 
     def fit_start(self, state: State, logger: Logger) -> None:
+        # Verify safety with self.overwrite. Note that this has to be done at fit_start as opposed to init since it requires state.timestamp
+        # from any checkpoints which are loaded, and checkpoint loading happens after Event.INIT.
+        if not self.overwrite:
+            folder = format_name_with_dist(self.folder, logger.run_name)
+            ensure_folder_has_no_conflicting_files(folder, self.filename, state.timestamp)
+        # Ensure no rank proceeds (and potentially attempts to write to the folder), until all ranks have validated that the folder is safe.
+        dist.barrier()
         if state.is_model_deepspeed:
             if self.weights_only:
                 NotImplementedError(
@@ -332,12 +340,16 @@ class CheckpointSaver(Callback):
     def batch_checkpoint(self, state: State, logger: Logger):
         if self.save_interval(state, Event.BATCH_CHECKPOINT):
             # If training is finished, log at the FIT loglevel
-            log_level = LogLevel.BATCH if state.get_elapsed_duration() < 1.0 else LogLevel.FIT
+            elapsed_duration = state.get_elapsed_duration()
+            assert elapsed_duration is not None, "elapsed_duration is set on Event.BATCH_CHECKPOINT"
+            log_level = LogLevel.BATCH if elapsed_duration < 1.0 else LogLevel.FIT
             self._save_checkpoint(state, logger, log_level)
 
     def epoch_checkpoint(self, state: State, logger: Logger):
         if self.save_interval(state, Event.EPOCH_CHECKPOINT):
-            log_level = LogLevel.EPOCH if state.get_elapsed_duration() < 1.0 else LogLevel.FIT
+            elapsed_duration = state.get_elapsed_duration()
+            assert elapsed_duration is not None, "elapsed_duration is set on Event.BATCH_CHECKPOINT"
+            log_level = LogLevel.EPOCH if elapsed_duration < 1.0 else LogLevel.FIT
             self._save_checkpoint(state, logger, log_level)
 
     def _save_checkpoint(self, state: State, logger: Logger, log_level: LogLevel):
@@ -352,7 +364,7 @@ class CheckpointSaver(Callback):
             checkpoint_filepath = checkpoint_filepaths[dist.get_global_rank()]
             if self.artifact_name is not None:
                 artifact_name = format_name_with_dist_and_time(self.artifact_name, logger.run_name,
-                                                               state.timer.get_timestamp()).lstrip("/")
+                                                               state.timestamp).lstrip("/")
                 if state.is_model_deepspeed and not is_tar(artifact_name):
                     # Deepspeed requires tarballs; appending `.tar`
                     artifact_name += ".tar"
@@ -362,10 +374,14 @@ class CheckpointSaver(Callback):
                                      overwrite=self.overwrite)
 
             if self.latest_filename is not None:
+                formatted_folder_path = format_name_with_dist(self.folder, logger.run_name)
                 symlink_name = os.path.join(
-                    format_name_with_dist(self.folder, logger.run_name),
-                    format_name_with_dist_and_time(self.latest_filename, logger.run_name,
-                                                   state.timer.get_timestamp()).lstrip("/"),
+                    formatted_folder_path,
+                    format_name_with_dist_and_time(
+                        self.latest_filename,
+                        logger.run_name,
+                        state.timestamp,
+                    ).lstrip("/"),
                 )
                 if state.is_model_deepspeed and not is_tar(symlink_name):
                     # Deepspeed requires tarballs; appending `.tar`
@@ -377,19 +393,20 @@ class CheckpointSaver(Callback):
                     os.remove(symlink_name)
                 except FileNotFoundError:
                     pass
-                os.symlink(checkpoint_filepath, symlink_name)
+                relative_checkpoint_path = os.path.relpath(checkpoint_filepath, formatted_folder_path)
+                os.symlink(relative_checkpoint_path, symlink_name)
                 if self.artifact_name is not None and self.latest_artifact_name is not None:
                     symlink_artifact_name = format_name_with_dist_and_time(self.latest_artifact_name, logger.run_name,
-                                                                           state.timer.get_timestamp()).lstrip("/")
+                                                                           state.timestamp).lstrip("/")
                     artifact_name = format_name_with_dist_and_time(self.artifact_name, logger.run_name,
-                                                                   state.timer.get_timestamp()).lstrip("/")
+                                                                   state.timestamp).lstrip("/")
                     # Always overwrite for symlinks since we use the same filename for latest
                     logger.symlink_artifact(log_level=log_level,
                                             existing_artifact_name=artifact_name,
                                             symlink_artifact_name=symlink_artifact_name,
                                             overwrite=True)
 
-        timestamp = state.timer.get_timestamp()
+        timestamp = state.timestamp
 
         self.saved_checkpoints.append((timestamp, checkpoint_filepaths))
         if self.num_checkpoints_to_keep >= 0:
