@@ -1,11 +1,12 @@
-# Copyright 2021 MosaicML. All Rights Reserved.
+# Copyright 2022 MosaicML Composer authors
+# SPDX-License-Identifier: Apache-2.0
 
 """Specifications for operating and training on data."""
 from __future__ import annotations
 
 import collections.abc
 import textwrap
-from typing import TYPE_CHECKING, Any, Callable, List, Mapping, Optional, Sequence, Tuple
+from typing import TYPE_CHECKING, Any, Callable, Iterable, List, Mapping, Optional, Sequence, Tuple, Union
 
 import torch
 import torch.utils.data
@@ -13,12 +14,12 @@ import torch.utils.data
 from composer.utils.iter_helpers import ensure_tuple
 
 if TYPE_CHECKING:
-    from composer.core.types import Batch, DataLoader
+    from composer.core.types import Batch
 
-__all__ = ["DataSpec"]
+__all__ = ["DataSpec", "ensure_data_spec"]
 
 
-def _split_list(l, num_microbatches):
+def _split_list(l, num_microbatches: int):
     if len(l) < num_microbatches:
         raise ValueError(
             textwrap.dedent(f"""\
@@ -27,7 +28,7 @@ def _split_list(l, num_microbatches):
     return [l[i::num_microbatches] for i in range(num_microbatches)]
 
 
-def _split_tensor(t, num_microbatches):
+def _split_tensor(t, num_microbatches: int):
     if len(t) < num_microbatches:
         raise ValueError(
             textwrap.dedent(f"""\
@@ -36,7 +37,7 @@ def _split_tensor(t, num_microbatches):
     return t.chunk(num_microbatches)
 
 
-def _split_mapping(m, num_microbatches):
+def _split_mapping(m, num_microbatches: int):
     chunked = {}
     for k, v in m.items():
         if isinstance(v, torch.Tensor):
@@ -91,7 +92,7 @@ class DataSpec:
 
     .. doctest::
 
-       >>> # In this case, we apply NormalizationFn 
+       >>> # In this case, we apply NormalizationFn
        >>> # Construct DataSpec as shown below to apply this transformation
        >>> from composer.datasets.utils import NormalizationFn
        >>> CHANNEL_MEAN = (0.485 * 255, 0.456 * 255, 0.406 * 255)
@@ -114,24 +115,24 @@ class DataSpec:
         trainer.engine.close()
 
     Args:
-        dataloader (DataLoader): The dataloader.
+        dataloader (Iterable): The dataloader, which can be any iterable that yields batches.
 
         num_samples (int, optional): The total number of samples in an epoch, across all ranks. This field is used by
-            the :class:`~.time.Timer` (training progress tracker). If not specified, then ``len(dataloader.dataset)`` is
+            the :class:`~.time.Timestamp` (training progress tracker). If not specified, then ``len(dataloader.dataset)`` is
             used (if this property is available). Otherwise, the dataset is assumed to be unsized.
 
         num_tokens (int, optional): The total number of tokens in an epoch. This field is used by the
-            :class:`~.time.Timer` (training progress tracker).
+            :class:`~.time.Timestamp` (training progress tracker).
 
         device_transforms ((Batch) -> Batch, optional): Function called by the :class:`~.trainer.Trainer` to modify the
             batch once it has been moved onto the device. For example, this function can be used for GPU-based
-            normalization.  It can modify the batch in-place, and it should return the modified batch. If not specified, the
-            batch is not modified.
+            normalization.  It can modify the batch in-place, and it should return the modified batch. If not specified,
+            the batch is not modified.
 
         split_batch ((Batch, int) -> Sequence[Batch], optional): Function called by the :class:`~.trainer.Trainer` to
-            split a batch (the first parameter) into the number of microbatches specified (the second parameter).  By
-            default, batches of type :attr:`~.types.BatchPair` can be split automatically. If the ``dataloader`` yields
-            batches of a different type, then this function must be specified.
+            split a batch (the first parameter) into the number of microbatches specified (the second parameter). If the
+            ``dataloader`` yields batches not of type torch.Tensor, Mapping, Tuple, or List, then this function must
+            be specified.
 
         get_num_samples_in_batch ((Batch) -> int, optional): Function that is called by the :class:`~.trainer.Trainer`
             to get the number of samples in the provided batch.
@@ -149,7 +150,7 @@ class DataSpec:
 
     def __init__(
         self,
-        dataloader: DataLoader,
+        dataloader: Iterable,
         num_samples: Optional[int] = None,
         num_tokens: Optional[int] = None,
         device_transforms: Optional[Callable[[Batch], Batch]] = None,
@@ -167,7 +168,6 @@ class DataSpec:
             self.num_samples = num_samples
 
         else:
-            # FFCV dataloaders don't have an associated dataset
             if isinstance(dataloader, torch.utils.data.DataLoader) and isinstance(dataloader.dataset,
                                                                                   collections.abc.Sized):
                 try:
@@ -177,6 +177,15 @@ class DataSpec:
             else:
                 self.num_samples = None
 
+        if isinstance(dataloader, torch.utils.data.DataLoader) and dataloader._iterator is not None:
+            raise ValueError(
+                ("The dataloader has an active iterator. This could occur "
+                 "if `persistent_workers=True` and the dataloader has already been iterated, "
+                 "or if the dataloader is mid-epoch. It is required that the training dataloader "
+                 "does not have an active iterator, so CPU dataset augmentations can be "
+                 "correctly inserted. To fix, please do not iterate over the dataloader before passing it into "
+                 "the Trainer."))
+
     def _default_device_transforms(self, batch: Batch):
         return batch
 
@@ -204,3 +213,21 @@ class DataSpec:
     def _default_get_num_tokens_in_batch(self, batch: Batch) -> int:
         del batch  # unused
         return 0
+
+
+def ensure_data_spec(dataloader: Union[DataSpec, Iterable, dict]) -> DataSpec:
+    """Ensures that the ``dataloader`` is a :class:`.DataSpec`
+
+    Args:
+        dataloader (DataSpec | Iterable | dict): A DataSpec, DataLoader, or Dict of DataSpec kwargs.
+
+    Returns:
+        DataSpec: A DataSpec
+    """
+    if isinstance(dataloader, dict):
+        # treat as kwargs for DataSpec
+        dataloader = DataSpec(**dataloader)
+    if not isinstance(dataloader, DataSpec):
+        dataloader = DataSpec(dataloader)
+
+    return dataloader
