@@ -1,13 +1,13 @@
-# Copyright 2021 MosaicML. All Rights Reserved.
+# Copyright 2022 MosaicML Composer authors
+# SPDX-License-Identifier: Apache-2.0
 
 """The state of the trainer."""
 from __future__ import annotations
 
-import contextlib
+import collections.abc
 import logging
-import textwrap
 import warnings
-from typing import TYPE_CHECKING, Any, Callable, ContextManager, Dict, List, Optional, Sequence, Union, cast
+from typing import TYPE_CHECKING, Any, Callable, Dict, Iterable, List, Optional, Sequence, Union, cast
 
 import torch
 import torch.nn.modules.utils
@@ -16,8 +16,8 @@ from torch.optim import Optimizer
 
 from composer.core.precision import Precision
 from composer.core.serializable import Serializable
-from composer.core.time import Time, Timer, TimeUnit
-from composer.utils import dist, ensure_tuple
+from composer.core.time import Time, Timestamp, TimeUnit
+from composer.utils import batch_get, batch_set, dist, ensure_tuple
 
 if TYPE_CHECKING:
     import deepspeed
@@ -33,24 +33,6 @@ __all__ = ["State"]
 logger = logging.getLogger(__name__)
 
 
-def _default_precision_factory() -> Callable[[Union[str, Precision]], ContextManager]:
-    """Returns a context manager to automatically cast to a specific precision.
-
-    Args:
-        precision (str or Precision): Precision for the context
-    """
-    if torch.cuda.is_available():
-        return lambda precision: torch.cuda.amp.autocast(Precision(precision) == Precision.AMP)
-    else:
-
-        def null(precision):
-            assert Precision(
-                precision) != Precision.AMP, "Precision AMP is only available when `torch.cuda.is_available() == True`."
-            return contextlib.nullcontext()
-
-        return null
-
-
 def _ensure_backwards_compatible_checkpointing(state_dict: Dict[str, Any]):
     # v0.4.1 removed the leading underscores for the keys in the state_dict
     # It also renamed _is_model_ddp_wrapped to is_model_ddp
@@ -73,7 +55,7 @@ _STATE_DICT_SERIALIZED_ATTRIBUTES = [
     "algorithms",
     "callbacks",
     "scaler",
-    "timer",
+    "timestamp",
 ]
 
 
@@ -95,18 +77,24 @@ class State(Serializable):
         model (torch.nn.Module): The model, typically as a subclass of :class:`~.ComposerModel`.
         rank_zero_seed (int): The seed used on the rank zero process. It is assumed that each rank's seed is
             ``rank_zero_seed + dist.get_global_rank()``.
-        grad_accum (int): The number of gradient accumulation steps to use. With this argument, micro batch size for
-            each device becomes ``microbatch_size = train_batch_size / (num_devices * grad_accum)``.
-        train_dataloader (types.DataLoader, DataSpec, or dict):
-            The :class:`~.types.DataLoader`, :class:`~.DataSpec`, or dict of :class:`~.DataSpec` kwargs to used for training.
-        evaluators (evaluator.Evaluator | Sequence[evaluator.Evaluator]):
-            The evaluators contain the evaluation dataset(s) used for evaluation with specific metrics.
-        max_duration (str or Time): The maximum duration to train for.
+        grad_accum (int, optional): The number of gradient accumulation steps to use. With this argument, micro batch
+            size for each device becomes ``microbatch_size = train_batch_size / (num_devices * grad_accum)``.
+        train_dataloader (types.DataLoader, optional): Dataloader used for training
+        evaluators (Evalutor | Evaluators, optional): :class:`.Evaluator` used for evaluation.
+        dataloader (types.DataLoader, optional): The active DataLoader.
+        dataloader_len (int | Time[int], optional): The number of batches per dataloader iteration (e.g. epoch).
+            The trainer will yield the first ``dataloader_len`` batches per iteration. If ``-1`` (the default),
+            the entire dataloader will be iterated over.
+        dataloader_label (str, optional): The name for the dataloader. Required if ``dataloader`` is specified.
+            (default: ``None``)
+
+            By convention, the training dataloader is called ``'train'``. The evaluator dataloader is called
+            ``'eval'``, or when multiple evaluators are used, the name of the evaluator.
+        max_duration (str | Time, optional): The maximum duration to train for. (default: ``None``)
         precision (str | Precision): The numerical precision to use for training. See :class:`~.Precision` for
             the supported precisions.
-        precision_context (Callable[[Precision], ContextManager]): Function to produce a context manager to mandate precision.
-        optimizers (torch.optim.Optimizer | Sequence[torch.optim.Optimizer], optional): The optimizer being used to train the model.
-            Multiple optimizers are not currently supported.
+        optimizers (torch.optim.Optimizer | Sequence[torch.optim.Optimizer], optional): The optimizer being used to
+            train the model. Multiple optimizers are not currently supported.
         schedulers (types.PyTorchScheduler | Sequence[types.PyTorchScheduler], optional):
             The learning rate scheduler (can also be a list or tuple of schedulers).
         scaler (torch.cuda.amp.GradScaler, optional): The gradient scaler in use for mixed precision training.
@@ -156,10 +144,28 @@ class State(Serializable):
             >>> trainer.fit()
             >>> trainer.state.current_metrics
             {'train': {'Accuracy': tensor(...)}, 'eval1': {'Accuracy': tensor(...)}, 'eval2': {'Accuracy': tensor(...)}}
-
+        eval_timestamp (Timestamp): The timestamp for the current evaluation dataloader. This timestamp is reset
+            before the dataloader is evaluated. The :attr:`~Timestamp.epoch` attribute for this timestamp is always
+            ``0``.
+        grad_accum (int): The number of gradient accumulation steps per batch.
         loss (torch.Tensor | Sequence[torch.Tensor]): The most recently computed loss.
-        outputs (torch.Tensor | Sequence[torch.Tensor]): The most recently computed output from the model's forward pass.
-        timer (Timer): The timer that tracks training loop progress.
+        model (torch.nn.Module): The training model.
+
+            .. note::
+
+                When using DeepSpeed or multi-rank training, the model will be wrapped with
+                :class:`~deepspeed.DeepSpeedEngine` or :class:`~torch.nn.parallel.DistributedDataParallel`,
+                respectively.
+
+        outputs (torch.Tensor | Sequence[torch.Tensor]): The most recently computed output from the model's forward
+            pass.
+        predict_timestamp (Timestamp): The timestamp for the current prediction dataloader. This timestamp is reset
+            before the dataloader is used. The :attr:`~Timestamp.epoch` attribute for this timestamp is always
+            ``0``.
+        profiler (Profiler): The profiler (if profiling is enabled), or ``None`` if not profiling.
+        rank_zero_seed (int): The seed of the rank zero process.
+        scaler (torch.cuda.amp.GradScaler): The gradient scaler if using mixed-precision training, or
+            ``None`` if not using mixed-precision training.
         serialized_attributes (List[str]): The names of the attribute which are serialized in a checkpoint.
 
             By default, the following attributes are serialized:
@@ -179,16 +185,22 @@ class State(Serializable):
             +-----------------------+-------------------------------------------------------------+
             | scaler                | The gradient scaler in use for mixed precision training.    |
             +-----------------------+-------------------------------------------------------------+
-            | timer                 | The timer that tracks training loop progress.               |
+            | timestamp             | The timestamp that tracks training loop progress.           |
             +-----------------------+-------------------------------------------------------------+
             | rank_zero_seed        | The seed of the rank zero process.                          |
             +-----------------------+-------------------------------------------------------------+
             | current_metrics       | The current metrics.                                        |
             +-----------------------+-------------------------------------------------------------+
+
+        timestamp (Timestamp): The current training timestamp.
+        train_dataloader (Iterable): The training dataloader. (May be ``None`` if not training.)
     """
 
-    _max_duration: Time[int]
-    _steps_per_epoch: Optional[int]
+    _dataloader: Optional[Iterable]
+    _dataloader_label: Optional[str]
+    _dataloader_len: Optional[Time[int]]
+    _max_duration: Optional[Time[int]]
+
     batch: types.Batch
     batch_num_samples: int
     batch_num_tokens: int
@@ -201,18 +213,27 @@ class State(Serializable):
         # model
         model: torch.nn.Module,
 
-        # stopping conditions
-        max_duration: Union[str, Time[int]],
+        # determinism
         rank_zero_seed: int,
 
+        # stopping conditions
+        max_duration: Optional[Union[str, Time[int]]] = None,
+
         # data configurations
-        train_dataloader: types.DataLoader,
-        evaluators: Optional[Union[Evaluator, Sequence[Evaluator]]] = None,
         grad_accum: int = 1,
 
+        # dataloaders
+        train_dataloader: Optional[Iterable] = None,
+        evaluators: Optional[Union[Evaluator, Sequence[Evaluator]]] = None,
+
+        # these track the current 'active' dataloader
+        # depending on train, eval, or others
+        dataloader: Optional[Iterable] = None,
+        dataloader_label: Optional[str] = None,
+        dataloader_len: Union[int, Time[int]] = -1,
+
         # precision
         precision: Union[str, Precision] = Precision.FP32,
-        precision_context: Callable[[Precision], ContextManager] = _default_precision_factory(),
 
         # optimizers
         optimizers: Optional[Union[Optimizer, Sequence[Optimizer]]] = None,
@@ -223,23 +244,21 @@ class State(Serializable):
         # algorithms and callbacks
         algorithms: Optional[Union[Algorithm, Sequence[Algorithm]]] = None,
         callbacks: Optional[Union[Callback, Sequence[Callback]]] = None,
-
-        # steps per epoch
-        steps_per_epoch: Optional[int] = None,
     ):
         self.rank_zero_seed = rank_zero_seed
         self.model = model
         self.grad_accum = grad_accum
-        self.train_dataloader = train_dataloader
+        self._dataloader_len = None
+        self.set_dataloader(dataloader, dataloader_label, dataloader_len)
         self.max_duration = max_duration
-        self.steps_per_epoch = steps_per_epoch
 
         self.train_dataloader = train_dataloader
         self._evaluators = list(ensure_tuple(evaluators))
 
-        self.timer = Timer()
+        self.timestamp = Timestamp()
+        self.eval_timestamp = Timestamp()
+        self.predict_timestamp = Timestamp()
         self._precision = Precision(precision)
-        self._precision_context = precision_context
 
         if optimizers is None:
             self._optimizers = []
@@ -253,6 +272,7 @@ class State(Serializable):
         self._callbacks = list(ensure_tuple(callbacks))
 
         self.profiler: Optional[Profiler] = None
+
         # These attributes will be serialized using .state_dict(), and loaded with .load_state_dict()
         # All other attributes will not be serialized.
         # For simplicity, omit the leading underscore for private attributes.
@@ -265,7 +285,7 @@ class State(Serializable):
             "algorithms",
             "callbacks",
             "scaler",
-            "timer",
+            "timestamp",
             "rank_zero_seed",
             "current_metrics",
         ]
@@ -283,24 +303,31 @@ class State(Serializable):
         return self._max_duration
 
     @max_duration.setter
-    def max_duration(self, max_duration: Union[str, Time[int]]):
+    def max_duration(self, max_duration: Optional[Union[str, Time[int]]]):
+        if max_duration is None:
+            self._max_duration = None
+            return
         if isinstance(max_duration, str):
             max_duration = cast(Time[int], Time.from_timestring(max_duration))
         if max_duration.unit == TimeUnit.DURATION:
             raise ValueError("TimeUnit.DURATION is not allowed as a unit for max_duration")
         self._max_duration = max_duration
 
-    def get_elapsed_duration(self) -> Time[float]:
+    def get_elapsed_duration(self) -> Optional[Time[float]]:
         """Get the elapsed training duration.
 
         Returns:
-            Time: The elapsed duration, in :attr:`TimeUnit.DURATION`. ``Time(0.0, TimeUnit.DURATION)`` represents the
-                beginning of training and ``Time(1.0, TimeUnit.DURATION)`` represents a completed training process.
+            Optional[Time[float]]: The elapsed duration, in :attr:`TimeUnit.DURATION`.
+                ``Time(0.0, TimeUnit.DURATION)`` represents the beginning of training and ``Time(1.0, TimeUnit.DURATION)``
+                represents a completed training process. Returns ``None`` if ``max_duration`` is None.
         """
-        return self.timer.get(self.max_duration.unit) / self.max_duration
+        if self.max_duration is None:
+            return None
+        return self.timestamp.get(self.max_duration.unit) / self.max_duration
 
     @property
     def optimizers(self):
+        """The optimizers."""
         return self._optimizers
 
     @optimizers.setter
@@ -309,14 +336,68 @@ class State(Serializable):
 
     @property
     def schedulers(self):
+        """The schedulers."""
         return self._schedulers
 
     @schedulers.setter
-    def schedulers(self, schedulers: types.PyTorchScheduler):
+    def schedulers(self, schedulers: Union[types.PyTorchScheduler, Sequence[types.PyTorchScheduler]]):
         self._schedulers[:] = ensure_tuple(schedulers)
 
+    def batch_get_item(self, key: Optional[Any] = None, get_fn: Optional[Callable[[Any], Any]] = None) -> Any:
+        """Gets element from batch either specified by key or user-specified function.
+
+        Args:
+            key (Any): A key to index into the batch. Key is optional if get_fn is supplied.
+            get_fn (Callable): A user-specified function to do the extracting. 
+                Note: get_fn is optional if key is supplied.
+
+        Returns:
+            The part of the batch specified by the key extracted by the get_fn. This could
+                be any type depending on what the batch is composed of.
+        
+        Raises:
+            ValueError if key is unset and get_fn is unset or if both are set.
+        """
+        if key is None and get_fn is None:
+            raise ValueError("key or get_fn must be specified and neither were!")
+        if key is not None and get_fn is not None:
+            raise ValueError("key and get_fn were both set. Only one can be set!")
+        return batch_get(self.batch, key, get_fn)
+
+    def batch_set_item(self,
+                       *,
+                       key: Optional[Any] = None,
+                       value: Any,
+                       set_fn: Optional[Callable[[Any, Any], Any]] = None):
+        """Sets the element specified by the key of the set_fn to the specified value. 
+
+        This is not an in-place operation, as for tuple-typed batches, a new batch object 
+        must be created to modify them.
+
+        Args:
+            key (Any): A key to index into the batch. Optional if set_fn is specified.
+            value (Any): The value that batch[key] or batch.key gets set to or that the 
+                set_fn uses to set a part of the batch to.
+            set_fn (Callable): A user-specified function to do the setting. set_fn is optional if key and 
+                value are supplied. The set_fn must return the updated batch.
+
+        Returns:
+            batch (Any): The updated batch with value set at key.
+
+        Raises:
+            ValueError if:
+                * key and set_fn are both unset
+                * key and set_fn are both set
+        """
+        if key is None and set_fn is None:
+            raise ValueError("key or set_fn must be specified and neither were!")
+        if key is not None and set_fn is not None:
+            raise ValueError("key and set_fn were both set. Only one can be set!")
+        self.batch = batch_set(self.batch, key=key, value=value, set_fn=set_fn)
+
     @property
     def callbacks(self):
+        """The callbacks."""
         return self._callbacks
 
     @callbacks.setter
@@ -325,6 +406,7 @@ class State(Serializable):
 
     @property
     def algorithms(self):
+        """The algorithms."""
         return self._algorithms
 
     @algorithms.setter
@@ -333,6 +415,7 @@ class State(Serializable):
 
     @property
     def evaluators(self):
+        """The evaluators."""
         return self._evaluators
 
     @evaluators.setter
@@ -340,7 +423,6 @@ class State(Serializable):
         self._evaluators[:] = list(ensure_tuple(evaluators))
 
     def state_dict(self) -> Dict[str, Any]:
-        """Returns the state as a :class:`dict`."""
         state_dict = {}
 
         for attribute_name in self.serialized_attributes:
@@ -365,7 +447,7 @@ class State(Serializable):
         return state_dict
 
     def load_model_state(self, state_dict: Dict[str, Any], strict: bool):
-        """Loads the model's state from a state_dict.
+        """Loads the model's state from a ``state_dict``.
 
         Args:
             state_dict (Dict[str, Any]): The state dict, generated from a previous call to :meth:`state_dict`.
@@ -420,25 +502,96 @@ class State(Serializable):
                     pass
 
     @property
-    def steps_per_epoch(self):
-        """int: The maximum number of steps (batches) per epoch."""
-        if self._steps_per_epoch is None:
-            return len(self.train_dataloader)
-        return self._steps_per_epoch
+    def dataloader(self):
+        """The active dataloader."""
+        return self._dataloader
 
-    @steps_per_epoch.setter
-    def steps_per_epoch(self, steps_per_epoch: Optional[int]):
+    @property
+    def dataloader_label(self):
+        """The dataloader label for the active dataloader.
+
+        By default, the training dataloader is called ``'train'``. The evaluator dataloader
+        is called ``'eval'``, or when multiple evaluators are used, the name of the evaluator.
+        However, the dataloader label can be explicitely specified in :meth:`.Trainer.fit`
+        and :meth:`.Trainer.eval`.
+
+        Returns:
+            Optional[str]: The dataloader label, or None if no dataloader is set.
+        """
+        return self._dataloader_label
+
+    def set_dataloader(
+        self,
+        dataloader: Optional[Iterable] = None,
+        dataloader_label: Optional[str] = None,
+        dataloader_len: Union[int, Time[int]] = -1,
+    ):
+        """Update the active dataloader and dataloader label.
+
+        Args:
+            dataloader (Iterable, optional): The dataloader. Defaults to None.
+            dataloader_label (str, optional): The dataloader label. Must be ``None`` if and only if
+                ``dataloader`` is None. Defaults to None.
+            dataloader_len (int, int): The number of batches per dataloader iteration (e.g. epoch), as used by the trainer.
+                Set to ``-1`` to iterate over the entire dataset. (Default: ``-1``.)
+        """
+        if dataloader is None:
+            dataloader_label = None
+        else:
+            if dataloader_label is None:
+                raise ValueError("If the `dataloader` is specified, then `dataloader_label` must not be None.")
+        self._dataloader = dataloader
+        self._dataloader_label = dataloader_label
+        if dataloader is not None:
+            self.dataloader_len = dataloader_len  # setting it to -1 will do a failsafe read of len(dataloader)
+        else:
+            self._dataloader_len = None
+
+    @property
+    def dataloader_len(self):
+        """The number of batches per dataloader iteration (e.g. epoch), as used by the trainer.
+
+        .. note::
+
+            If not explicitely specified, this value is an approximation, as it depends on ``len(self.dataloader)``.
+            See the :doc:`PyTorch DataLoader Documentation <torch:data>` for more information.
+
+        Returns:
+            Optional[Time[int]]: The number of batches per dataloader iteration (e.g. epoch), or None if no dataloader
+            is defined or if the dataloader has an unknown length (e.g. streaming dataloaders).
+        """
+        return self._dataloader_len
+
+    @dataloader_len.setter
+    def dataloader_len(self, num_batches: Union[int, Time[int]]):
+        if isinstance(num_batches, int):
+            num_batches = Time(num_batches, TimeUnit.BATCH)
+        if self._dataloader is None:
+            raise RuntimeError("`State.dataloader_len` cannot be set if the dataloader is not defined.")
         try:
-            dataloader_len = len(self.train_dataloader)
+            if isinstance(self._dataloader, collections.abc.Sized):
+                dataloader_len = len(self._dataloader)
+            else:
+                dataloader_len = None
         except (TypeError, NotImplementedError):
             dataloader_len = None
-        if dataloader_len is not None and steps_per_epoch is not None and steps_per_epoch > dataloader_len:
-            warnings.warn(
-                textwrap.dedent(f"""\
-                    SubsetNumBatchesWarning: The steps_per_epoch({steps_per_epoch})
-                    is greater than the number of batches in the training dataloader
-                    ({dataloader_len})"""))
-        self._steps_per_epoch = steps_per_epoch
+        if dataloader_len is not None and num_batches >= 0 and int(num_batches) > dataloader_len:
+            warnings.warn((f"DataloaderNumBatchesWarning: The dataloader_len ({int(num_batches)}) "
+                           f"is greater than the length (i.e. number of batches) of the dataloader, which is "
+                           f"{dataloader_len}. State.dataloader_len is thus being set to {dataloader_len}."))
+            self._dataloader_len = Time(dataloader_len, TimeUnit.BATCH)
+            return
+        if num_batches < 0:
+            if dataloader_len is not None:
+                # len(dataloader) is an approximation -- see https://pytorch.org/docs/stable/data.html.
+                # However, in the worst case where additional last batches are dropped, this calculation should be
+                # an over-estimate, leading to the entire dataloader still being iterated over.
+                self._dataloader_len = Time(dataloader_len, TimeUnit.BATCH)
+            else:
+                # The dataloader length is unknown.
+                self._dataloader_len = None
+            return
+        self._dataloader_len = num_batches
 
     @property
     def precision(self):
@@ -452,30 +605,6 @@ class State(Serializable):
     def precision(self, precision: Union[str, Precision]):
         self._precision = Precision(precision)
 
-    @property
-    def batch_pair(self) -> types.BatchPair:
-        """:attr:`~.types.BatchPair`: The current batch, represented as a :attr:`~.types.BatchPair`.
-
-        Raises:
-            TypeError: If the current batch is not a :attr:`~.types.BatchPair`.
-        """
-        from composer.core.types import as_batch_pair
-        return as_batch_pair(self.batch)
-
-    @property
-    def batch_dict(self) -> types.BatchDict:
-        """:attr:`~.types.BatchDict`: The current batch, represented as a :attr:`~.types.BatchDict`.
-
-        Raises:
-            TypeError: If the current batch is not a :attr:`~.types.BatchDict`.
-        """
-        from composer.core.types import as_batch_dict
-        return as_batch_dict(self.batch)
-
-    @property
-    def precision_context(self):
-        return self._precision_context(self.precision)
-
     @property
     def is_model_deepspeed(self) -> bool:
         """Whether :attr:`model` is an instance of a :class:`~deepspeed.DeepSpeedEngine`."""
