@@ -1,4 +1,5 @@
-# Copyright 2021 MosaicML. All Rights Reserved.
+# Copyright 2022 MosaicML Composer authors
+# SPDX-License-Identifier: Apache-2.0
 
 """MNIST image classification dataset.
 
@@ -11,7 +12,6 @@ from dataclasses import dataclass
 import yahp as hp
 from torchvision import datasets, transforms
 
-from composer.core.types import DataLoader
 from composer.datasets.dataloader import DataLoaderHparams
 from composer.datasets.hparams import DatasetHparams, SyntheticHparamsMixin, WebDatasetHparams
 from composer.datasets.synthetic import SyntheticBatchPairDataset
@@ -30,7 +30,7 @@ class MNISTDatasetHparams(DatasetHparams, SyntheticHparamsMixin):
     """
     download: bool = hp.optional("whether to download the dataset, if needed", default=True)
 
-    def initialize_object(self, batch_size: int, dataloader_hparams: DataLoaderHparams) -> DataLoader:
+    def initialize_object(self, batch_size: int, dataloader_hparams: DataLoaderHparams):
         if self.use_synthetic:
             dataset = SyntheticBatchPairDataset(
                 total_dataset_size=60_000 if self.is_train else 10_000,
@@ -71,7 +71,7 @@ class MNISTWebDatasetHparams(WebDatasetHparams):
     remote: str = hp.optional('WebDataset S3 bucket name', default='s3://mosaicml-internal-dataset-mnist')
     name: str = hp.optional('WebDataset local cache name', default='mnist')
 
-    def initialize_object(self, batch_size: int, dataloader_hparams: DataLoaderHparams) -> DataLoader:
+    def initialize_object(self, batch_size: int, dataloader_hparams: DataLoaderHparams):
         from composer.datasets.webdataset_utils import load_webdataset
 
         split = 'train' if self.is_train else 'val'
