@@ -1,4 +1,5 @@
-# Copyright 2021 MosaicML. All Rights Reserved.
+# Copyright 2022 MosaicML Composer authors
+# SPDX-License-Identifier: Apache-2.0
 
 """Logger Hyperparameter classes.
 
@@ -132,7 +133,7 @@ class WandBLoggerHparams(LoggerDestinationHparams):
             config_dict = self.extra_init_params["config"]
 
         if self.flatten_config:
-            config_dict = self._flatten_dict(config_dict)
+            config_dict = self._flatten_dict(config_dict, prefix=[])
 
         init_params = {
             "project": self.project,
@@ -150,7 +151,7 @@ class WandBLoggerHparams(LoggerDestinationHparams):
         )
 
     @classmethod
-    def _flatten_dict(cls, data: Dict[str, Any], _prefix: List[str] = []) -> Dict[str, Any]:
+    def _flatten_dict(cls, data: Dict[str, Any], prefix: List[str]) -> Dict[str, Any]:
         """Flattens a dictionary with list or sub dicts to have dot syntax.
 
         .. testcode::
@@ -175,7 +176,7 @@ class WandBLoggerHparams(LoggerDestinationHparams):
         """
         all_items = {}
         for key, val in data.items():
-            key_items = _prefix + [key]
+            key_items = list(prefix) + [key]
             key_name = ".".join(key_items)
             if isinstance(val, dict):
                 all_items.update(cls._flatten_dict(val, key_items))
@@ -234,7 +235,7 @@ class InMemoryLoggerHparams(LoggerDestinationHparams):
     hyperparameters.
 
     Args:
-        log_level (str or LogLevel, optional):
+        log_level (str | LogLevel, optional):
             See :class:`~composer.loggers.in_memory_logger.InMemoryLogger`.
     """
     log_level: LogLevel = hp.optional("The maximum verbosity to log. Default: BATCH", default=LogLevel.BATCH)
