@@ -1,3 +1,6 @@
+# Copyright 2022 MosaicML Composer authors
+# SPDX-License-Identifier: Apache-2.0
+
 """Single Shot Object Detection model with pretrained ResNet34 backbone extending :class:`.ComposerModel`."""
 
 import os
@@ -9,7 +12,6 @@ import requests
 from torch import Tensor
 from torchmetrics import Metric, MetricCollection
 
-from composer.core.types import BatchPair
 from composer.models.base import ComposerModel
 from composer.models.ssd.base_model import Loss
 from composer.models.ssd.ssd300 import SSD300
@@ -60,7 +62,7 @@ class SSD(ComposerModel):
         from composer.datasets.coco import COCODetection
         self.val_coco = COCODetection(val_coco_root, val_annotate, val_trans)
 
-    def loss(self, outputs: Any, batch: BatchPair) -> Union[Tensor, Sequence[Tensor]]:
+    def loss(self, outputs: Any, batch: Any) -> Union[Tensor, Sequence[Tensor]]:
 
         (_, _, _, bbox, label) = batch  #type: ignore
         if not isinstance(bbox, Tensor):
@@ -76,12 +78,12 @@ class SSD(ComposerModel):
     def metrics(self, train: bool = False) -> Union[Metric, MetricCollection]:
         return self.MAP
 
-    def forward(self, batch: BatchPair) -> Tensor:
+    def forward(self, batch: Any) -> Tensor:
         (img, _, _, _, _) = batch  #type: ignore
         ploc, plabel = self.module(img)
         return ploc, plabel  #type: ignore
 
-    def validate(self, batch: BatchPair) -> Tuple[Any, Any]:
+    def validate(self, batch: Any) -> Tuple[Any, Any]:
         inv_map = {v: k for k, v in self.val_coco.label_map.items()}
         ret = []
         overlap_threshold = self.overlap_threshold
