@@ -1,11 +1,12 @@
+# Copyright 2022 MosaicML Composer authors
+# SPDX-License-Identifier: Apache-2.0
+
 """A convenience class that creates a :class:`.ComposerModel` for classification tasks from a vanilla PyTorch model.
 
 :class:`.ComposerClassifier` requires batches in the form: (``input``, ``target``) and includes a basic
 classification training loop with :func:`.soft_cross_entropy` loss and accuracy logging.
 """
 
-from __future__ import annotations
-
 import logging
 from typing import Any, Optional, Tuple, Union
 
@@ -14,7 +15,6 @@ from torch import Tensor
 from torchmetrics import Metric, MetricCollection
 from torchmetrics.classification import Accuracy
 
-from composer.core.types import BatchPair
 from composer.loss import loss_registry
 from composer.metrics import CrossEntropy
 from composer.models import ComposerModel
@@ -33,7 +33,7 @@ class ComposerClassifier(ComposerModel):
         module (torch.nn.Module): A PyTorch neural network module.
         loss_name (str, optional): Loss function to use. E.g. 'soft_cross_entropy' or
             'binary_cross_entropy_with_logits'. Loss function must be in
-            :mod:`~composer.loss.loss`. Default: ``'soft_cross_entropy'``".      
+            :mod:`~composer.loss.loss`. Default: ``'soft_cross_entropy'``".
 
     Returns:
         ComposerClassifier: An instance of :class:`.ComposerClassifier`.
@@ -72,7 +72,7 @@ class ComposerClassifier(ComposerModel):
                         "Please ensure you are using `initializers. "
                         "linear_log_constant_bias`.")
 
-    def loss(self, outputs: Any, batch: BatchPair, *args, **kwargs) -> Tensor:
+    def loss(self, outputs: Any, batch: Any, *args, **kwargs) -> Tensor:
         _, targets = batch
         if not isinstance(outputs, Tensor):  # to pass typechecking
             raise ValueError("Loss expects input as Tensor")
@@ -83,12 +83,12 @@ class ComposerClassifier(ComposerModel):
     def metrics(self, train: bool = False) -> Union[Metric, MetricCollection]:
         return self.train_acc if train else MetricCollection([self.val_acc, self.val_loss])
 
-    def forward(self, batch: BatchPair) -> Tensor:
+    def forward(self, batch: Tuple[Tensor, Tensor]) -> Tensor:
         inputs, _ = batch
         outputs = self.module(inputs)
         return outputs
 
-    def validate(self, batch: BatchPair) -> Tuple[Any, Any]:
+    def validate(self, batch: Any) -> Tuple[Any, Any]:
         _, targets = batch
         outputs = self.forward(batch)
         return outputs, targets
