@@ -1,4 +1,5 @@
-# Copyright 2021 MosaicML. All Rights Reserved.
+# Copyright 2022 MosaicML Composer authors
+# SPDX-License-Identifier: Apache-2.0
 
 """Stateless learning rate schedulers.
 
@@ -36,7 +37,7 @@ __all__ = [
     "ComposerScheduler", "compile_composer_scheduler", "StepScheduler", "MultiStepScheduler", "ConstantScheduler",
     "LinearScheduler", "ExponentialScheduler", "CosineAnnealingScheduler", "CosineAnnealingWarmRestartsScheduler",
     "PolynomialScheduler", "MultiStepWithWarmupScheduler", "LinearWithWarmupScheduler",
-    "CosineAnnealingWithWarmupScheduler"
+    "CosineAnnealingWithWarmupScheduler", "PolynomialWithWarmupScheduler"
 ]
 
 
@@ -51,7 +52,7 @@ class ComposerScheduler(Protocol):
     .. code:: python
 
         def ten_epoch_decay_scheduler(state: State) -> float:
-            if state.timer.epoch < 10:
+            if state.timestamp.epoch < 10:
                 return 1.0
             return 0.5
 
@@ -94,7 +95,7 @@ class ComposerScheduler(Protocol):
 
         A scheduler function should be a pure function that returns a multiplier to apply to the optimizer's provided
         learning rate, given the current trainer state, and optionally a "scale schedule ratio" (SSR). A typical
-        implementation will read ``state.timer``, and possibly other fields like ``state.max_duration``, to determine
+        implementation will read ``state.timestamp``, and possibly other fields like ``state.max_duration``, to determine
         the trainer's latest temporal progress.
 
         .. note::
@@ -107,7 +108,7 @@ class ComposerScheduler(Protocol):
         .. note::
             It is possible to use multiple schedulers, in which case their effects will stack multiplicatively.
 
-        The ``ssr`` param indicates that the schedule should be "stretched" accordingly. In symbolic terms, where 
+        The ``ssr`` param indicates that the schedule should be "stretched" accordingly. In symbolic terms, where
         :math:`\alpha_\sigma(t)` represents the scheduler output at time :math:`t` using scale schedule ratio
         :math:`\sigma`:
 
@@ -130,16 +131,22 @@ def _convert_time(time: Union[str, Time[int], Time[float]], state: State, ssr: f
     if isinstance(time, str):
         time = Time.from_timestring(time)
 
+    assert state.max_duration is not None, "max_duration should be set whenever schedulers are invoked"
+
     if time.unit == TimeUnit.DURATION:
+        if state.dataloader_len is None:
+            raise RuntimeError("Cannot convert time, as state.dataloader_len is None.")
         if state.max_duration.unit == TimeUnit.EPOCH:
-            return Time(int(time.value * state.steps_per_epoch * state.max_duration.value), TimeUnit.BATCH)
+            return Time(int(time.value * int(state.dataloader_len) * state.max_duration.value), TimeUnit.BATCH)
         return Time(int(time.value * state.max_duration.value), state.max_duration.unit)
 
     if time.unit == TimeUnit.EPOCH:
         # Epochs do not provide sufficient granularity for SSR scaling
         # e.g. if max_duration = 1ep, then any SSR would result in a new duration of 0.
         # so, convert the time into batches
-        time = Time(value=time.value * state.steps_per_epoch, unit=TimeUnit.BATCH)
+        if state.dataloader_len is None:
+            raise RuntimeError("Cannot convert time, as state.dataloader_len is None.")
+        time = Time(value=time.value * int(state.dataloader_len), unit=TimeUnit.BATCH)
 
     return Time(value=int(time.value * ssr), unit=time.unit)
 
@@ -202,9 +209,9 @@ class StepScheduler(ComposerScheduler):
 
     Where :math:`\rho` represents the time between changes to the learning rate (the step size), and
     :math:`\gamma` represents the multiplicative decay factor.
-    
+
     Args:
-        step_size (str or Time): Time between changes to the learning rate.
+        step_size (str | Time): Time between changes to the learning rate.
         gamma (float): Multiplicative decay factor. Default = ``0.1``.
     """
 
@@ -214,7 +221,7 @@ class StepScheduler(ComposerScheduler):
 
     def __call__(self, state: State, ssr: float = 1.0):
         step_size = _convert_time(self.step_size, state, ssr=ssr)
-        current_time = state.timer.get(step_size.unit)
+        current_time = state.timestamp.get(step_size.unit)
         steps = int(current_time / step_size)
 
         return self.gamma**steps
@@ -235,9 +242,9 @@ class MultiStepScheduler(ComposerScheduler):
 
     Where :math:`x` represents the amount of milestones that have been reached, and :math:`\gamma` represents the
     multiplicative decay factor.
-    
+
     Args:
-        milestones (List[str or Time]): Times at which the learning rate should change.
+        milestones (List[str | Time]): Times at which the learning rate should change.
         gamma (float): Multiplicative decay factor. Default = ``0.1``.
     """
 
@@ -250,7 +257,7 @@ class MultiStepScheduler(ComposerScheduler):
 
         factor = 1.0
         for milestone in milestones:
-            if state.timer >= milestone:
+            if state.timestamp >= milestone:
                 factor *= self.gamma
 
         return factor
@@ -271,10 +278,10 @@ class ConstantScheduler(ComposerScheduler):
 
     Where :math:`\alpha` represents the learning rate multiplier to maintain while this scheduler is active, and
     :math:`t_{max}` represents the duration of this scheduler.
-    
+
     Args:
         alpha (float): Learning rate multiplier to maintain while this scheduler is active. Default = ``1.0``.
-        t_max (str or Time): Duration of this scheduler. Default = ``"1dur"``.
+        t_max (str | Time): Duration of this scheduler. Default = ``"1dur"``.
     """
 
     def __init__(self, alpha: float = 1.0, t_max: Union[str, Time] = "1dur") -> None:
@@ -284,7 +291,7 @@ class ConstantScheduler(ComposerScheduler):
     def __call__(self, state: State, ssr: float = 1.0) -> float:
         t_max = _convert_time(self.t_max, state, ssr=ssr)
 
-        if state.timer < t_max:
+        if state.timestamp < t_max:
             return self.alpha
 
         return 1.0
@@ -299,7 +306,7 @@ class LinearScheduler(ComposerScheduler):
     .. warning::
         Note that the defaults for this scheduler differ from the defaults for
         :class:`~torch.optim.lr_scheduler.LinearLR`. The PyTorch scheduler, by default, linearly increases the learning
-        rate multiplier from 1.0 / 3 to 1.0, whereas this implementation, by default, linearly decreases the multiplier 
+        rate multiplier from 1.0 / 3 to 1.0, whereas this implementation, by default, linearly decreases the multiplier
         rom 1.0 to 0.0.
 
     Linearly adjusts the learning rate multiplier from ``alpha_i`` to ``alpha_f`` over ``t_{max}`` time.
@@ -313,14 +320,14 @@ class LinearScheduler(ComposerScheduler):
 
     .. math::
         \tau = t / t_{max}
-    
+
     Where :math:`\alpha_i` represents the initial learning rate multiplier, :math:`\alpha_f` represents
     the learning rate multiplier to decay to, and :math:`t_{max}` represents the duration of this scheduler.
-    
+
     Args:
         alpha_i (float): Initial learning rate multiplier. Default = ``1.0``.
         alpha_f (float): Final learning rate multiplier. Default = ``0.0``.
-        t_max (str or Time): The duration of this scheduler. Default = ``"1dur"``.
+        t_max (str | Time): The duration of this scheduler. Default = ``"1dur"``.
     """
 
     def __init__(self, alpha_i: float = 1.0, alpha_f: float = 0.0, t_max: Union[str, Time] = "1dur"):
@@ -330,7 +337,7 @@ class LinearScheduler(ComposerScheduler):
 
     def __call__(self, state: State, ssr: float = 1.0):
         t_max = _convert_time(self.t_max, state, ssr=ssr)
-        current_time = state.timer.get(t_max.unit)
+        current_time = state.timestamp.get(t_max.unit)
         frac_of_total = min(1.0, (current_time / t_max).value)
 
         current_factor = self.alpha_i + frac_of_total * (self.alpha_f - self.alpha_i)
@@ -352,9 +359,9 @@ class ExponentialScheduler(ComposerScheduler):
         \alpha(t) = \gamma ^ {t / \rho}
 
     Where :math:`\rho` represents the decay period, and :math:`\gamma` represents the multiplicative decay factor.
-    
+
     Args:
-        decay_period (str or Time): Decay period. Default = ``"1ep"``.
+        decay_period (str | Time): Decay period. Default = ``"1ep"``.
         gamma (float): Multiplicative decay factor.
     """
 
@@ -364,7 +371,7 @@ class ExponentialScheduler(ComposerScheduler):
 
     def __call__(self, state: State, ssr: float = 1.0):
         decay_period = _convert_time(self.decay_period, state, ssr)
-        current_time_in_decay_units = state.timer.get(decay_period.unit)
+        current_time_in_decay_units = state.timestamp.get(decay_period.unit)
 
         return self.gamma**float(current_time_in_decay_units / decay_period)
 
@@ -395,12 +402,12 @@ class CosineAnnealingScheduler(ComposerScheduler):
 
     .. math::
         \tau = t / t_{max}
-    
+
     Where :math:`t_{max}`
     represents the duration of this scheduler, and :math:`\alpha_f` represents the learning rate multiplier to decay to.
-    
+
     Args:
-        t_max (str or Time): The duration of this scheduler. Default = ``"1dur"``.
+        t_max (str | Time): The duration of this scheduler. Default = ``"1dur"``.
         alpha_f (float): Learning rate multiplier to decay to. Default = ``0.0``.
     """
 
@@ -410,7 +417,7 @@ class CosineAnnealingScheduler(ComposerScheduler):
 
     def __call__(self, state: State, ssr: float = 1.0):
         t_max = _convert_time(self.t_max, state, ssr=ssr)
-        current_time = state.timer.get(t_max.unit)
+        current_time = state.timestamp.get(t_max.unit)
         frac_of_total = (current_time / t_max).value
 
         return _cosine_anneal(x=frac_of_total, min_y=self.alpha_f)
@@ -435,13 +442,13 @@ class CosineAnnealingWarmRestartsScheduler(ComposerScheduler):
 
     .. math::
         \tau_i = (t - \sum_{j=0}^{i-1} t_0 t_{mult}^j) / (t_0 t_{mult}^i)
-    
+
     Where :math:`t_0`
     represents the period of the first cycle, :math:`t_{mult}` represents the multiplier for the duration of successive
     cycles, and :math:`\alpha_f` represents the learning rate multiplier to decay to.
-    
+
     Args:
-        t_0 (str or Time): The period of the first cycle.
+        t_0 (str | Time): The period of the first cycle.
         t_mult (float): The multiplier for the duration of successive cycles. Default = ``1.0``.
         alpha_f (float): Learning rate multiplier to decay to. Default = ``0.0``.
     """
@@ -455,7 +462,7 @@ class CosineAnnealingWarmRestartsScheduler(ComposerScheduler):
         t_0 = _convert_time(self.t_0, state, ssr=ssr)
         current_interval_len = t_0
         current_interval_end = t_0
-        while current_interval_end <= state.timer.get(current_interval_end.unit):
+        while current_interval_end <= state.timestamp.get(current_interval_end.unit):
             if current_interval_len.value == 0:
                 raise ValueError(
                     "Interval between restarts for cosine annealing/warm restarts scheduler has decayed to 0.")
@@ -465,7 +472,8 @@ class CosineAnnealingWarmRestartsScheduler(ComposerScheduler):
             current_interval_end += current_interval_len
 
         current_interval_start = current_interval_end - current_interval_len
-        frac_of_current_interval = ((state.timer.get(t_0.unit) - current_interval_start) / current_interval_len).value
+        frac_of_current_interval = ((state.timestamp.get(t_0.unit) - current_interval_start) /
+                                    current_interval_len).value
 
         return _cosine_anneal(x=frac_of_current_interval, min_y=self.alpha_f)
 
@@ -482,14 +490,14 @@ class PolynomialScheduler(ComposerScheduler):
 
     .. math::
         \tau = t / t_{max}
-    
+
     Where :math:`\kappa`
     represents the exponent to be used for the proportionality relationship, :math:`t_{max}` represents the duration of
     this scheduler, and :math:`\alpha_f` represents the learning rate multiplier to decay to.
 
     Args:
         power (float): The exponent to be used for the proportionality relationship.
-        t_max (str or Time): The duration of this scheduler. Default = ``"1dur"``.
+        t_max (str | Time): The duration of this scheduler. Default = ``"1dur"``.
         alpha_f (float): Learning rate multiplier to decay to. Default = ``0.0``.
     """
 
@@ -500,7 +508,7 @@ class PolynomialScheduler(ComposerScheduler):
 
     def __call__(self, state: State, ssr: float = 1.0):
         t_max = _convert_time(self.t_max, state, ssr=ssr)
-        current_time = state.timer.get(t_max.unit)
+        current_time = state.timestamp.get(t_max.unit)
         frac_of_total = (current_time / t_max).value
 
         coeff = (1 - frac_of_total)**self.power
@@ -537,8 +545,8 @@ class MultiStepWithWarmupScheduler(ComposerScheduler):
         will still be scaled accordingly.
 
     Args:
-        t_warmup (str or Time): Warmup time.
-        milestones (List[str or Time]): Times at which the learning rate should change.
+        t_warmup (str | Time): Warmup time.
+        milestones (List[str | Time]): Times at which the learning rate should change.
         gamma (float): Multiplicative decay factor. Default = ``0.1``.
     """
 
@@ -558,7 +566,7 @@ class MultiStepWithWarmupScheduler(ComposerScheduler):
                 training duration, take note that the warmup duration is calculated in the
                 same unit as the trainer's max_duration parameter."""))
 
-        if state.timer < t_warmup:
+        if state.timestamp < t_warmup:
             return self.warmup_scheduler(state)
 
         return self.step_scheduler(state, ssr)
@@ -575,7 +583,7 @@ class LinearWithWarmupScheduler(ComposerScheduler):
     Specifically, the learning rate multiplier :math:`\alpha` can be expressed as:
 
     .. math::
-        \alpha(t) = \begin{cases}    
+        \alpha(t) = \begin{cases}
             t / t_{warmup}, & \text{if } t < t_{warmup} \\
             \alpha_i + (alpha_f - \alpha_i) \times \tau_w & \text{otherwise}
         \end{cases}
@@ -584,7 +592,7 @@ class LinearWithWarmupScheduler(ComposerScheduler):
 
     .. math::
         \tau_w = (t - t_{warmup}) / t_{max}
-    
+
     Where :math:`t_{warmup}` represents the warmup time, :math:`\alpha_i` represents the initial learning rate multiplier,
     and :math:`\alpha_f` represents the learning rate multiplier to decay to, and :math:`t_{max}` represents the duration
     of this scheduler.
@@ -595,10 +603,10 @@ class LinearWithWarmupScheduler(ComposerScheduler):
         slightly distorted from what would otherwise be expected.
 
     Args:
-        t_warmup (str or Time): Warmup time.
+        t_warmup (str | Time): Warmup time.
         alpha_i (float): Initial learning rate multiplier. Default = ``1.0``.
         alpha_f (float): Final learning rate multiplier. Default = ``0.0``.
-        t_max (str or Time): The duration of this scheduler. Default = ``"1dur"``.
+        t_max (str | Time): The duration of this scheduler. Default = ``"1dur"``.
     """
 
     def __init__(self,
@@ -621,11 +629,11 @@ class LinearWithWarmupScheduler(ComposerScheduler):
                 training duration, take note that the warmup duration is calculated in the
                 same unit as the trainer's max_duration parameter."""))
 
-        if state.timer < t_warmup:
+        if state.timestamp < t_warmup:
             return self.warmup_scheduler(state)
 
         t_max = _convert_time(self.t_max, state, ssr=ssr)
-        current_time = state.timer.get(t_warmup.unit)
+        current_time = state.timestamp.get(t_warmup.unit)
         frac_of_total = min(1.0, ((current_time - t_warmup) / (t_max - t_warmup)).value)
 
         current_factor = self.alpha_i + frac_of_total * (self.alpha_f - self.alpha_i)
@@ -642,7 +650,7 @@ class CosineAnnealingWithWarmupScheduler(ComposerScheduler):
     Specifically, the learning rate multiplier :math:`\alpha` can be expressed as:
 
     .. math::
-        \alpha(t) = \begin{cases}    
+        \alpha(t) = \begin{cases}
             t / t_{warmup}, & \text{if } t < t_{warmup} \\
             \alpha_f + (1 - \alpha_f) \times \frac{1}{2} (1 + \cos(\pi \times \tau_w)) & \text{otherwise}
         \end{cases}
@@ -651,7 +659,7 @@ class CosineAnnealingWithWarmupScheduler(ComposerScheduler):
 
     .. math::
        \tau_w = (t - t_{warmup}) / t_{max}
-    
+
     Where :math:`t_{warmup}` represents the warmup time, :math:`t_{max}` represents the duration of this scheduler, and
     :math:`\alpha_f` represents the learning rate multiplier to decay to.
 
@@ -659,10 +667,10 @@ class CosineAnnealingWithWarmupScheduler(ComposerScheduler):
         Initial warmup time is **not** scaled according to any provided scale schedule ratio! However, the duration of
         the scheduler is still scaled accordingly. To achieve this, after warmup, the scheduler's "pace" will be
         slightly distorted from what would otherwise be expected.
-    
+
     Args:
-        t_warmup (str or Time): Warmup time.
-        t_max (str or Time): The duration of this scheduler. Default = ``"1dur"``.
+        t_warmup (str | Time): Warmup time.
+        t_max (str | Time): The duration of this scheduler. Default = ``"1dur"``.
         alpha_f (float): Learning rate multiplier to decay to. Default = ``0.0``.
     """
 
@@ -681,11 +689,78 @@ class CosineAnnealingWithWarmupScheduler(ComposerScheduler):
                 training duration, take note that the warmup duration is calculated in the
                 same unit as the trainer's max_duration parameter."""))
 
-        if state.timer < t_warmup:
+        if state.timestamp < t_warmup:
             return self.warmup_scheduler(state)
 
         t_max = _convert_time(self.t_max, state, ssr=ssr)
-        current_time = state.timer.get(t_warmup.unit)
+        current_time = state.timestamp.get(t_warmup.unit)
         frac_of_total = ((current_time - t_warmup) / (t_max - t_warmup)).value
 
         return _cosine_anneal(x=frac_of_total, min_y=self.alpha_f)
+
+
+class PolynomialWithWarmupScheduler(ComposerScheduler):
+    r"""Decays the learning rate according to a power of the fraction of training time left, with an initial warmup.
+
+    .. seealso::
+        This scheduler is based on :class:`~.PolynomialScheduler`, with an added warmup.
+
+    Specifically, the learning rate multiplier :math:`\alpha` can be expressed as:
+
+    .. math::
+        \alpha(t) = \begin{cases}
+            t / t_{warmup}, & \text{if } t < t_{warmup} \\
+            \alpha_f + (1 - \alpha_f) \times (1 - \tau_w) ^ {\kappa} & \text{otherwise}
+        \end{cases}
+
+    Given :math:`\tau_w`, the fraction of post-warmup time elpased (clipped to the interval :math:`[0, 1]`), as:
+
+    .. math::
+       \tau_w = (t - t_{warmup}) / t_{max}
+
+    Where :math:`\kappa` represents the exponent to be used for the proportionality relationship, 
+    :math:`t_{warmup}` represents the warmup time, :math:`t_{max}` represents the duration of this scheduler, and
+    :math:`\alpha_f` represents the learning rate multiplier to decay to.
+
+    .. warning::
+        Initial warmup time is **not** scaled according to any provided scale schedule ratio! However, the duration of
+        the scheduler is still scaled accordingly. To achieve this, after warmup, the scheduler's "pace" will be
+        slightly distorted from what would otherwise be expected.
+
+    Args:
+        t_warmup (str | Time): Warmup time.
+        power (float): The exponent to be used for the proportionality relationship. Default = ``2.0``.
+        t_max (str | Time): The duration of this scheduler. Default = ``"1dur"``.
+        alpha_f (float): Learning rate multiplier to decay to. Default = ``0.0``.
+    """
+
+    def __init__(self,
+                 t_warmup: Union[str, Time],
+                 power: float = 2.0,
+                 t_max: Union[str, Time] = "1dur",
+                 alpha_f: float = 0.0):
+        self.t_warmup = t_warmup
+        self.power = power
+        self.t_max = t_max
+        self.alpha_f = alpha_f
+        self.warmup_scheduler = LinearScheduler(alpha_i=0.0, alpha_f=1.0, t_max=t_warmup)
+
+    def __call__(self, state: State, ssr: float = 1.0):
+        t_warmup = _convert_time(self.t_warmup, state)
+        if t_warmup.value == 0:
+            warnings.warn(
+                textwrap.dedent("""\
+                The warmup duration is 0. If you specified warmup as a fraction of total
+                training duration, take note that the warmup duration is calculated in the
+                same unit as the trainer's max_duration parameter."""))
+
+        if state.timestamp < t_warmup:
+            return self.warmup_scheduler(state)
+
+        t_max = _convert_time(self.t_max, state, ssr=ssr)
+        current_time = state.timestamp.get(t_warmup.unit)
+        frac_of_total = ((current_time - t_warmup) / (t_max - t_warmup)).value
+
+        coeff = (1 - frac_of_total)**self.power
+        current_factor = self.alpha_f + coeff * (1.0 - self.alpha_f)
+        return current_factor
