@@ -1,10 +1,12 @@
-# Copyright 2021 MosaicML. All Rights Reserved.
+# Copyright 2022 MosaicML Composer authors
+# SPDX-License-Identifier: Apache-2.0
 
 """Helper utilities."""
+from composer.utils.batch_helpers import batch_get, batch_set
 from composer.utils.checkpoint import load_checkpoint, save_checkpoint
-from composer.utils.file_helpers import (ensure_folder_is_empty, format_name_with_dist, format_name_with_dist_and_time,
-                                         get_file, is_tar)
 from composer.utils.collect_env import configure_excepthook, disable_env_report, enable_env_report, print_env
+from composer.utils.file_helpers import (ensure_folder_has_no_conflicting_files, ensure_folder_is_empty,
+                                         format_name_with_dist, format_name_with_dist_and_time, get_file, is_tar)
 from composer.utils.import_helpers import MissingConditionalImportError, import_object
 from composer.utils.iter_helpers import ensure_tuple, iterate_with_pbar, map_collection
 from composer.utils.object_store import ObjectStore, ObjectStoreHparams
@@ -23,9 +25,12 @@ __all__ = [
     "load_checkpoint",
     "save_checkpoint",
     'ensure_folder_is_empty',
+    'ensure_folder_has_no_conflicting_files',
     'format_name_with_dist',
     'format_name_with_dist_and_time',
     'is_tar',
+    'batch_get',
+    'batch_set',
     'configure_excepthook',
     'disable_env_report',
     'enable_env_report',
